"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convert = convert;
exports.revert = revert;
exports.default = exports.UPPERCASE_TRANSFORMER = exports.TITLECASE_TRANSFORMER = exports.SENTENCECASE_TRANSFORMER = exports.LOWERCASE_TRANSFORMER = void 0;
var COMBINING_CHARS = /[\u0300-\u036F\u1AB0-\u1AFF\u1DC0-\u1DFF]+/g;
var INVALID_SEPARATOR = /[^-._~]/;
var CAMELCASE_PATTERN = '(?:[a-z](?=[A-Z])|[A-Z](?=[A-Z][a-z]))';
var CONVERT = /[A-Za-z\d]+/g;
var CONVERT_CAMELCASE = new RegExp("[A-Za-z0-9]*?".concat(CAMELCASE_PATTERN, "|[A-Za-z0-9]+"), 'g');
var REVERT = /[^-._~]+/g;
var REVERT_CAMELCASE = new RegExp("[^-._~]*?".concat(CAMELCASE_PATTERN, "|[^-._~]+"), 'g');
var REVERT_CAMELCASE_ONLY = new RegExp(".*?".concat(CAMELCASE_PATTERN, "|.+"), 'g');
/**
 * Validate options
 */

function validate(_ref) {
  var camelCase = _ref.camelCase,
      separator = _ref.separator,
      transformer = _ref.transformer;
  var extra = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (camelCase !== undefined) {
    if (typeof camelCase !== 'boolean') {
      throw new Error("camelCase must be a boolean: \"".concat(camelCase, "\"."));
    }
  }

  if (separator !== undefined && !('separator' in extra && separator === extra.separator)) {
    if (typeof separator !== 'string') {
      var error = 'separator' in extra ? " or ".concat(extra.separator) : '';
      throw new Error("separator must be a string".concat(error, ": \"").concat(separator, "\"."));
    } else if (INVALID_SEPARATOR.test(separator)) {
      var _error = separator.match(INVALID_SEPARATOR)[0];
      throw new Error("separator has an invalid character: \"".concat(_error, "\"."));
    }
  }

  if (transformer !== undefined) {
    if (transformer !== false && typeof transformer !== 'function') {
      var _error2 = "transformer must be false or a function: \"".concat(transformer, "\".");

      throw new Error(_error2);
    }
  }
}
/**
 * Builtin transformers
 */


var LOWERCASE_TRANSFORMER = function LOWERCASE_TRANSFORMER(fragments, separator) {
  return fragments.join(separator).toLowerCase();
};

exports.LOWERCASE_TRANSFORMER = LOWERCASE_TRANSFORMER;

var SENTENCECASE_TRANSFORMER = function SENTENCECASE_TRANSFORMER(fragments, separator) {
  var sentence = fragments.join(separator);
  return sentence.charAt(0).toUpperCase() + sentence.slice(1).toLowerCase();
};

exports.SENTENCECASE_TRANSFORMER = SENTENCECASE_TRANSFORMER;

var TITLECASE_TRANSFORMER = function TITLECASE_TRANSFORMER(fragments, separator) {
  return fragments.map(function (fragment) {
    return fragment.charAt(0).toUpperCase() + fragment.slice(1).toLowerCase();
  }).join(separator);
};

exports.TITLECASE_TRANSFORMER = TITLECASE_TRANSFORMER;

var UPPERCASE_TRANSFORMER = function UPPERCASE_TRANSFORMER(fragments, separator) {
  return fragments.join(separator).toUpperCase();
};
/**
 * Converts a string into a slug
 */


exports.UPPERCASE_TRANSFORMER = UPPERCASE_TRANSFORMER;

function convert(string) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  validate(options);
  var _options$camelCase = options.camelCase,
      camelCase = _options$camelCase === void 0 ? true : _options$camelCase,
      _options$separator = options.separator,
      separator = _options$separator === void 0 ? '-' : _options$separator,
      _options$transformer = options.transformer,
      transformer = _options$transformer === void 0 ? LOWERCASE_TRANSFORMER : _options$transformer;
  var fragments = String(string).normalize('NFKD').replace(COMBINING_CHARS, '').match(camelCase ? CONVERT_CAMELCASE : CONVERT);

  if (!fragments) {
    return '';
  }

  return transformer ? transformer(fragments, separator) : fragments.join(separator);
}
/**
 * Reverts a slug back to a string
 */


function revert(slug) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  validate(options, {
    separator: null
  });
  var _options$camelCase2 = options.camelCase,
      camelCase = _options$camelCase2 === void 0 ? false : _options$camelCase2,
      separator = options.separator,
      _options$transformer2 = options.transformer,
      transformer = _options$transformer2 === void 0 ? false : _options$transformer2;
  var fragments;
  slug = String(slug);
  /* Determine which method will be used split the slug */

  if ('' === separator) {
    fragments = camelCase ? slug.match(REVERT_CAMELCASE_ONLY) : [String(slug)];
  } else if ('string' === typeof separator) {
    fragments = slug.split(separator);
  } else {
    fragments = slug.match(camelCase ? REVERT_CAMELCASE : REVERT);
  }

  if (!fragments) {
    return '';
  }

  return transformer ? transformer(fragments, ' ') : fragments.join(' ');
}
/**
 * Sets convert() as the default export
 */


var _default = convert;
exports.default = _default;